// dbclass - ODBC/SQL database wrapper class, (c) 2000 by Tom Grandgent
//
// This code may not be used in commercial products without the express 
// written consent of Tom Grandgent (tom@grandgent.com)

#ifndef DBCLASS_INCLUDED
#define DBCLASS_INCLUDED

#include <sql.h>
#include <sqlext.h>

#define QUERY_READ					0
#define QUERY_WRITE					1

#define DBCLASS_MAX_COL_NAME_LENGTH	255
#define DBCLASS_MAX_COLS			64

void default_error_handler(char *text);

class Database
{
	friend class Recordset;

public:
	// Constructors and destructor
	Database(void);	// default constructor
	Database(char *sourcename);
	Database(char *sourcename, char *username, char *password);
	~Database(void); // destructor

	// Error handler functions
	void set_error_handler(void (*error_handler)(char *text));

	// Open/close a database
	int open(char *a_sourcename);
	int open(char *a_sourcename, char *a_username, char *a_password);
	void close(void);

	RETCODE error;

private:
	void init_odbc(void);
	void shutdown_odbc(void);
	void handle_error(SQLHSTMT hstmt, RETCODE error, char *text, ...);

	char datasource[128];
	char username[128];
	char password[128];
	HENV henv;	// handle to ODBC environment 
	HDBC hdbc;		// handle to database connection

	// Function pointer for app-defined error handler
	void (*error_handler)(char *text);

	// Variables for sql_error()
	unsigned char szSqlState[256];
	signed long pfNativeError;
	unsigned char szErrorMsg[SQL_MAX_MESSAGE_LENGTH];
	signed short cbErrorMsgMax;
};

class Recordset
{
	friend class Database;

public:
	// Constructors and destructor
	Recordset(Database &db);	// default constructor
	~Recordset(void); // destructor

	// Do a query
	int query(char *a_sql_statement, ...);
	int parameter(short int datatype, void *dataptr, int datalen);
	int named_parameter(char *name, short int datatype, void *dataptr, int datalen);
	int enumerate_columns(void);
	int parameter_column(int column, short int datatype, void *dataptr, int datalen);
	int execute_query(void);
	int col_count(void);
	int row_count(void);

	RETCODE error;

private:
	void handle_error(char *text, ...);

	// Parent database
	Database *parent;

	// Query data
	HSTMT hstmt;	// handle to an SQL statement (query)
	char *sql_statement;
	int query_type;
	int parameters;

	// Column data
	SQLSMALLINT num_columns;
	char col_name[DBCLASS_MAX_COLS][DBCLASS_MAX_COL_NAME_LENGTH];
	SQLSMALLINT col_type[DBCLASS_MAX_COLS];
	SQLUINTEGER col_size[DBCLASS_MAX_COLS];
	SQLSMALLINT col_decimal[DBCLASS_MAX_COLS];
	SQLSMALLINT col_nullable[DBCLASS_MAX_COLS];
	int cols_enumerated;
};

#endif